@@ -34,17 +34,17 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
-#include <libmissioncontrol/mission-control.h>
 #include <telepathy-glib/util.h>
 
+#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-log-manager.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-dispatcher.h>
 
 #include "empathy-chat.h"
 #include "empathy-conf.h"
 #include "empathy-spell.h"
-#include "empathy-spell-dialog.h"
 #include "empathy-contact-list-store.h"
 #include "empathy-contact-list-view.h"
 #include "empathy-contact-menu.h"
@@ -71,7 +71,7 @@ typedef struct {
 	EmpathyContact    *remote_contact;
 
 	EmpathyLogManager *log_manager;
-	MissionControl    *mc;
+	EmpathyAccountManager *account_manager;
 	GSList            *sent_messages;
 	gint               sent_messages_index;
 	GList             *compositors;
@@ -79,7 +79,6 @@ typedef struct {
 	guint              composing_stop_timeout_id;
 	guint              block_events_timeout_id;
 	TpHandleType       handle_type;
-	gpointer           token;
 	gint               contacts_width;
 	gboolean           has_input_vscroll;
 
@@ -167,35 +166,60 @@ chat_set_property (GObject      *object,
 }
 
 static void
-chat_status_changed_cb (MissionControl           *mc,
-			TpConnectionStatus        status,
-			McPresence                presence,
-			TpConnectionStatusReason  reason,
-			const gchar              *unique_name,
-			EmpathyChat              *chat)
+chat_connect_channel_reconnected (EmpathyDispatchOperation *dispatch,
+				  const GError             *error,
+				  gpointer                  user_data)
 {
-	EmpathyChatPriv *priv = GET_PRIV (chat);
-	McAccount       *account;
+	EmpathyChat *chat = EMPATHY_CHAT (user_data);
+	EmpathyTpChat *tpchat;
 
-	account = mc_account_lookup (unique_name);
-
-	if (status == TP_CONNECTION_STATUS_CONNECTED && !priv->tp_chat &&
-	    empathy_account_equal (account, priv->account) &&
-	    priv->handle_type != TP_HANDLE_TYPE_NONE) {
-		TpConnection *connection;
-
-		DEBUG ("Account reconnected, request a new Text channel");
-		connection = mission_control_get_tpconnection (mc, account, NULL);
-		tp_connection_run_until_ready (connection, FALSE, NULL, NULL);
-		empathy_connection_request_channel (connection, -1,
-						    TP_IFACE_CHANNEL_TYPE_TEXT,
-						    priv->handle_type,
-						    priv->id, TRUE,
-						    NULL, NULL, NULL, NULL);
-		g_object_unref (connection);
+	if (error != NULL) {
+		empathy_chat_view_append_event (chat->view,
+			_("Failed to reconnect this chat"));
+		return;
 	}
 
-	g_object_unref (account);
+	tpchat = EMPATHY_TP_CHAT (
+		empathy_dispatch_operation_get_channel_wrapper (dispatch));
+
+	if (empathy_dispatch_operation_claim (dispatch)) {
+		empathy_chat_set_tp_chat (chat, tpchat);
+	}
+}
+
+static void
+chat_new_connection_cb (EmpathyAccountManager *manager,
+			TpConnection *connection,
+			EmpathyChat *chat)
+{
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+	McAccount *account;
+
+	account = empathy_account_manager_get_account (manager, connection);
+	if (!priv->tp_chat && empathy_account_equal (account, priv->account) &&
+	    priv->handle_type != TP_HANDLE_TYPE_NONE &&
+	    !EMP_STR_EMPTY (priv->id)) {
+		
+		DEBUG ("Account reconnected, request a new Text channel");
+
+		switch (priv->handle_type) {
+			case TP_HANDLE_TYPE_CONTACT:
+				empathy_dispatcher_chat_with_contact_id (
+					connection, priv->id,
+					chat_connect_channel_reconnected,
+					chat);
+				break;
+			case TP_HANDLE_TYPE_ROOM:
+				empathy_dispatcher_join_muc (connection,
+					priv->id,
+					chat_connect_channel_reconnected,
+					chat);
+				break;
+			default:
+				g_assert_not_reached ();
+				break;
+		}
+	}
 }
 
 static void
@@ -350,12 +374,12 @@ chat_send (EmpathyChat  *chat,
 	EmpathyChatPriv *priv;
 	EmpathyMessage  *message;
 
-	priv = GET_PRIV (chat);
-
-	if (G_STR_EMPTY (msg)) {
+	if (EMP_STR_EMPTY (msg)) {
 		return;
 	}
 
+	priv = GET_PRIV (chat);
+
 	chat_sent_message_add (chat, msg);
 
 	if (g_str_has_prefix (msg, "/clear")) {
@@ -363,10 +387,19 @@ chat_send (EmpathyChat  *chat,
 		return;
 	}
 
+	/* Blacklist messages begining by '/', except for "/me" and "/say"
+	 * because they are handled in EmpathyMessage */
+	if (msg[0] == '/' &&
+	    !g_str_has_prefix (msg, "/me") &&
+	    !g_str_has_prefix (msg, "/say")) {
+		empathy_chat_view_append_event (chat->view,
+			_("Unsupported command"));
+		return;
+	}
+
+	/* We can send the message */
 	message = empathy_message_new (msg);
-
 	empathy_tp_chat_send (priv->tp_chat, message);
-
 	g_object_unref (message);
 }
 
@@ -454,9 +487,7 @@ chat_state_changed_cb (EmpathyTpChat      *tp_chat,
 }
 
 static void
-chat_message_received_cb (EmpathyTpChat  *tp_chat,
-			  EmpathyMessage *message,
-			  EmpathyChat    *chat)
+chat_message_received (EmpathyChat *chat, EmpathyMessage *message)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	EmpathyContact  *sender;
@@ -467,25 +498,25 @@ chat_message_received_cb (EmpathyTpChat  *tp_chat,
 		empathy_contact_get_name (sender),
 		empathy_contact_get_handle (sender));
 
-	if (priv->id) {
-		gboolean is_chatroom;
-
-		is_chatroom = priv->handle_type == TP_HANDLE_TYPE_ROOM;
-		empathy_log_manager_add_message (priv->log_manager,
-						 priv->id, is_chatroom,
-						 message);
-	}
-
 	empathy_chat_view_append_message (chat->view, message);
 
-	/* We received a message so the contact is no more composing */
-	chat_state_changed_cb (tp_chat, sender,
+	/* We received a message so the contact is no longer composing */
+	chat_state_changed_cb (priv->tp_chat, sender,
 			       TP_CHANNEL_CHAT_STATE_ACTIVE,
 			       chat);
 
 	g_signal_emit (chat, signals[NEW_MESSAGE], 0, message);
 }
 
+static void
+chat_message_received_cb (EmpathyTpChat  *tp_chat,
+			  EmpathyMessage *message,
+			  EmpathyChat    *chat)
+{
+	chat_message_received (chat, message);
+	empathy_tp_chat_acknowledge_message (tp_chat, message);
+}
+
 static void
 chat_send_error_cb (EmpathyTpChat          *tp_chat,
 		    EmpathyMessage         *message,
@@ -536,7 +567,7 @@ chat_property_changed_cb (EmpathyTpChat *tp_chat,
 		priv->subject = g_value_dup_string (value);
 		g_object_notify (G_OBJECT (chat), "subject");
 
-		if (G_STR_EMPTY (priv->subject)) {
+		if (EMP_STR_EMPTY (priv->subject)) {
 			gtk_widget_hide (priv->hbox_topic);
 		} else {
 			gtk_label_set_text (GTK_LABEL (priv->label_topic), priv->subject);
@@ -545,7 +576,7 @@ chat_property_changed_cb (EmpathyTpChat *tp_chat,
 		if (priv->block_events_timeout_id == 0) {
 			gchar *str;
 
-			if (!G_STR_EMPTY (priv->subject)) {
+			if (!EMP_STR_EMPTY (priv->subject)) {
 				str = g_strdup_printf (_("Topic set to: %s"), priv->subject);
 			} else {
 				str = g_strdup (_("No topic defined"));
@@ -561,31 +592,9 @@ chat_property_changed_cb (EmpathyTpChat *tp_chat,
 	}
 }
 
-static gboolean
-chat_get_is_command (const gchar *str)
-{
-	g_return_val_if_fail (str != NULL, FALSE);
-
-	if (str[0] != '/') {
-		return FALSE;
-	}
-
-	if (g_str_has_prefix (str, "/me")) {
-		return TRUE;
-	}
-	else if (g_str_has_prefix (str, "/nick")) {
-		return TRUE;
-	}
-	else if (g_str_has_prefix (str, "/topic")) {
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
 static void
 chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer,
-				   EmpathyChat    *chat)
+                                   EmpathyChat    *chat)
 {
 	EmpathyChatPriv *priv;
 	GtkTextIter     start, end;
@@ -601,8 +610,8 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer,
 	}
 
 	empathy_conf_get_bool (empathy_conf_get (),
-			      EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED,
-			      &spell_checker);
+                           EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED,
+                           &spell_checker);
 
 	gtk_text_buffer_get_start_iter (buffer, &start);
 
@@ -642,8 +651,8 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer,
 
 		str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
 
-		/* spell check string */
-		if (!chat_get_is_command (str)) {
+		/* spell check string if not a command */
+		if (str[0] != '/') {
 			correct = empathy_spell_check (str);
 		} else {
 			correct = TRUE;
@@ -921,13 +930,42 @@ chat_spell_free (EmpathyChatSpell *chat_spell)
 }
 
 static void
-chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem,
-				  EmpathyChatSpell *chat_spell)
+chat_spelling_menu_activate_cb (GtkMenuItem     *menu_item,
+				                EmpathyChatSpell *chat_spell)
 {
-	empathy_spell_dialog_show (chat_spell->chat,
-				  &chat_spell->start,
-				  &chat_spell->end,
-				  chat_spell->word);
+    empathy_chat_correct_word (chat_spell->chat,
+                               &(chat_spell->start),
+                               &(chat_spell->end),
+                               gtk_menu_item_get_label (menu_item));
+}
+
+static GtkWidget*
+chat_spelling_build_menu (EmpathyChatSpell *chat_spell)
+{
+    GtkWidget *menu, *menu_item;
+    GList     *suggestions, *l;
+
+    menu = gtk_menu_new ();
+    suggestions = empathy_spell_get_suggestions (chat_spell->word);
+    if (suggestions == NULL) {
+        menu_item = gtk_menu_item_new_with_label (_("(No Suggestions)"));
+	gtk_widget_set_sensitive (menu_item, FALSE);
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+    } else {
+        for (l = suggestions; l; l = l->next) {
+            menu_item = gtk_menu_item_new_with_label (l->data);
+            g_signal_connect (G_OBJECT (menu_item),
+                          "activate",
+                          G_CALLBACK (chat_spelling_menu_activate_cb),
+                          chat_spell);
+            gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+        }
+    }
+    empathy_spell_free_suggestions (suggestions);
+
+    gtk_widget_show_all (menu);
+
+    return menu;
 }
 
 static void
@@ -950,7 +988,8 @@ chat_input_populate_popup_cb (GtkTextView *view,
 	GtkTextIter           iter, start, end;
 	GtkWidget            *item;
 	gchar                *str = NULL;
-	EmpathyChatSpell      *chat_spell;
+	EmpathyChatSpell     *chat_spell;
+	GtkWidget            *spell_menu;
 	EmpathySmileyManager *smiley_manager;
 	GtkWidget            *smiley_menu;
 	GtkWidget            *image;
@@ -970,7 +1009,7 @@ chat_input_populate_popup_cb (GtkTextView *view,
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
 
-	smiley_manager = empathy_smiley_manager_new ();
+	smiley_manager = empathy_smiley_manager_dup_singleton ();
 	smiley_menu = empathy_smiley_menu_new (smiley_manager,
 					       chat_insert_smiley_activate_cb,
 					       chat);
@@ -980,7 +1019,7 @@ chat_input_populate_popup_cb (GtkTextView *view,
 	/* Add the Send menu item. */
 	gtk_text_buffer_get_bounds (buffer, &start, &end);
 	str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-	if (!G_STR_EMPTY (str)) {
+	if (!EMP_STR_EMPTY (str)) {
 		item = gtk_menu_item_new_with_mnemonic (_("_Send"));
 		g_signal_connect (G_OBJECT (item), "activate",
 				  G_CALLBACK (chat_text_send_cb), chat);
@@ -1005,7 +1044,7 @@ chat_input_populate_popup_cb (GtkTextView *view,
 		str = gtk_text_buffer_get_text (buffer,
 						&start, &end, FALSE);
 	}
-	if (!G_STR_EMPTY (str)) {
+	if (!EMP_STR_EMPTY (str)) {
 		chat_spell = chat_spell_new (chat, str, start, end);
 		g_object_set_data_full (G_OBJECT (menu),
 					"chat_spell", chat_spell,
@@ -1015,27 +1054,44 @@ chat_input_populate_popup_cb (GtkTextView *view,
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
 
-		item = gtk_image_menu_item_new_with_mnemonic (_("_Check Word Spelling..."));
+		item = gtk_image_menu_item_new_with_mnemonic (_("_Spelling Suggestions"));
 		image = gtk_image_new_from_icon_name (GTK_STOCK_SPELL_CHECK,
 						      GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-		g_signal_connect (item,
-				  "activate",
-				  G_CALLBACK (chat_text_check_word_spelling_cb),
-				  chat_spell);
+
+		spell_menu = chat_spelling_build_menu (chat_spell);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), spell_menu);
+
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
 	}
 }
 
+static gboolean
+chat_log_filter (EmpathyMessage *message,
+		 gpointer user_data)
+{
+	EmpathyChat *chat = (EmpathyChat *) user_data;
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+	const GList *pending;
+
+	pending = empathy_tp_chat_get_pending_messages (priv->tp_chat);
+
+	for (; pending; pending = g_list_next (pending)) {
+		if (empathy_message_equal (message, pending->data)) {
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
 static void
 chat_add_logs (EmpathyChat *chat)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	gboolean         is_chatroom;
 	GList           *messages, *l;
-	guint            num_messages;
-	guint            i;
 
 	if (!priv->id) {
 		return;
@@ -1046,25 +1102,20 @@ chat_add_logs (EmpathyChat *chat)
 
 	/* Add messages from last conversation */
 	is_chatroom = priv->handle_type == TP_HANDLE_TYPE_ROOM;
-	messages = empathy_log_manager_get_last_messages (priv->log_manager,
-							  priv->account,
-							  priv->id,
-							  is_chatroom);
-	num_messages  = g_list_length (messages);
 
-	/* Only keep the 10 last messages */
-	for (i = 0; num_messages - i > 10; i++) {
-		EmpathyMessage *message;
+	messages = empathy_log_manager_get_filtered_messages (priv->log_manager,
+							      priv->account,
+							      priv->id,
+							      is_chatroom,
+							      5,
+							      chat_log_filter,
+							      chat);
 
-		message = messages->data;
-		messages = g_list_remove (messages, message);
-		g_object_unref (message);
-	}
-
-	for (l = messages; l; l = l->next) {
+	for (l = messages; l; l = g_list_next (l)) {
 		empathy_chat_view_append_message (chat->view, l->data);
 		g_object_unref (l->data);
 	}
+
 	g_list_free (messages);
 
 	/* Turn back on scrolling */
@@ -1116,10 +1167,6 @@ chat_members_changed_cb (EmpathyTpChat  *tp_chat,
 	if (priv->block_events_timeout_id == 0) {
 		gchar *str;
 
-		empathy_contact_run_until_ready (contact,
-						 EMPATHY_CONTACT_READY_NAME,
-						 NULL);
-
 		if (is_member) {
 			str = g_strdup_printf (_("%s has joined the room"),
 					       empathy_contact_get_name (contact));
@@ -1234,6 +1281,7 @@ chat_destroy_cb (EmpathyTpChat *tp_chat,
 		return;
 	}
 
+	chat_composing_remove_timeout (chat);
 	g_object_unref (priv->tp_chat);
 	priv->tp_chat = NULL;
 	g_object_notify (G_OBJECT (chat), "tp-chat");
@@ -1243,20 +1291,35 @@ chat_destroy_cb (EmpathyTpChat *tp_chat,
 	chat_set_show_contacts (chat, FALSE);
 }
 
+static void
+show_pending_messages (EmpathyChat *chat) {
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+	const GList *messages, *l;
+
+	if (chat->view == NULL || priv->tp_chat == NULL)
+		return;
+
+	messages = empathy_tp_chat_get_pending_messages (priv->tp_chat);
+
+	for (l = messages; l != NULL ; l = g_list_next (l)) {
+		EmpathyMessage *message = EMPATHY_MESSAGE (l->data);
+		chat_message_received (chat, message);
+	}
+	empathy_tp_chat_acknowledge_messages (priv->tp_chat, messages);
+}
+
 static void
 chat_create_ui (EmpathyChat *chat)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
-	GladeXML        *glade;
+	GtkBuilder      *gui;
  	GList           *list = NULL; 
 	gchar           *filename;
 	GtkTextBuffer   *buffer;
 
-	filename = empathy_file_lookup ("empathy-chat.glade",
+	filename = empathy_file_lookup ("empathy-chat.ui",
 					"libempathy-gtk");
-	glade = empathy_glade_get_file (filename,
-					"chat_widget",
-					NULL,
+	gui = empathy_builder_get_file (filename,
 					"chat_widget", &priv->widget,
 					"hpaned", &priv->hpaned,
 					"vbox_left", &priv->vbox_left,
@@ -1267,7 +1330,6 @@ chat_create_ui (EmpathyChat *chat)
 					"scrolled_window_contacts", &priv->scrolled_window_contacts,
 					NULL);
 	g_free (filename);
-	g_object_unref (glade);
 
 	/* Add message view. */
 	chat->view = empathy_theme_manager_create_view (empathy_theme_manager_get ());
@@ -1333,6 +1395,7 @@ chat_create_ui (EmpathyChat *chat)
 
 	/* Add the main widget in the chat widget */
 	gtk_container_add (GTK_CONTAINER (chat), priv->widget);
+	g_object_unref (gui);
 }
 
 static void
@@ -1394,11 +1457,28 @@ chat_finalize (GObject *object)
 
 	chat_composing_remove_timeout (chat);
 
-	empathy_disconnect_account_status_changed (priv->token);
-	g_object_unref (priv->mc);
+	g_signal_handlers_disconnect_by_func (priv->account_manager,
+					      chat_new_connection_cb, object);
+
+	g_object_unref (priv->account_manager);
 	g_object_unref (priv->log_manager);
 
 	if (priv->tp_chat) {
+		g_signal_handlers_disconnect_by_func (priv->tp_chat,
+			chat_destroy_cb, chat);
+		g_signal_handlers_disconnect_by_func (priv->tp_chat,
+			chat_message_received_cb, chat);
+		g_signal_handlers_disconnect_by_func (priv->tp_chat,
+			chat_send_error_cb, chat);
+		g_signal_handlers_disconnect_by_func (priv->tp_chat,
+			chat_state_changed_cb, chat);
+		g_signal_handlers_disconnect_by_func (priv->tp_chat,
+			chat_property_changed_cb, chat);
+		g_signal_handlers_disconnect_by_func (priv->tp_chat,
+			chat_members_changed_cb, chat);
+		g_signal_handlers_disconnect_by_func (priv->tp_chat,
+			chat_remote_contact_changed_cb, chat);
+		empathy_tp_chat_close (priv->tp_chat);
 		g_object_unref (priv->tp_chat);
 	}
 	if (priv->account) {
@@ -1415,6 +1495,7 @@ chat_finalize (GObject *object)
 	g_free (priv->id);
 	g_free (priv->name);
 	g_free (priv->subject);
+	g_completion_free (priv->completion);
 
 	G_OBJECT_CLASS (empathy_chat_parent_class)->finalize (object);
 }
@@ -1426,6 +1507,7 @@ chat_constructed (GObject *object)
 
 	chat_create_ui (chat);
 	chat_add_logs (chat);
+	show_pending_messages (chat);
 }
 
 static void
@@ -1526,15 +1608,16 @@ empathy_chat_init (EmpathyChat *chat)
 		EMPATHY_TYPE_CHAT, EmpathyChatPriv);
 
 	chat->priv = priv;
-	priv->log_manager = empathy_log_manager_new ();
+	priv->log_manager = empathy_log_manager_dup_singleton ();
 	priv->contacts_width = -1;
 	priv->sent_messages = NULL;
 	priv->sent_messages_index = -1;
-	priv->mc = empathy_mission_control_new ();
+	priv->account_manager = empathy_account_manager_dup_singleton ();
 
-	priv->token = empathy_connect_to_account_status_changed (priv->mc,
-						   G_CALLBACK (chat_status_changed_cb),
-						   chat, NULL);
+	g_signal_connect (priv->account_manager,
+			  "new-connection",
+			  G_CALLBACK (chat_new_connection_cb),
+			  chat);
 
 	/* Block events for some time to avoid having "has come online" or
 	 * "joined" messages. */
@@ -1567,6 +1650,7 @@ empathy_chat_set_tp_chat (EmpathyChat   *chat,
 			  EmpathyTpChat *tp_chat)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
+	TpConnection    *connection;
 
 	g_return_if_fail (EMPATHY_IS_CHAT (chat));
 	g_return_if_fail (EMPATHY_IS_TP_CHAT (tp_chat));
@@ -1581,8 +1665,14 @@ empathy_chat_set_tp_chat (EmpathyChat   *chat,
 	}
 
 	priv->tp_chat = g_object_ref (tp_chat);
-	priv->account = g_object_ref (empathy_tp_chat_get_account (tp_chat));
+	connection = empathy_tp_chat_get_connection (priv->tp_chat);
+	priv->account = empathy_account_manager_get_account (priv->account_manager,
+							     connection);
+	g_object_ref (priv->account);
 
+	g_signal_connect (tp_chat, "destroy",
+			  G_CALLBACK (chat_destroy_cb),
+			  chat);
 	g_signal_connect (tp_chat, "message-received",
 			  G_CALLBACK (chat_message_received_cb),
 			  chat);
@@ -1601,9 +1691,6 @@ empathy_chat_set_tp_chat (EmpathyChat   *chat,
 	g_signal_connect_swapped (tp_chat, "notify::remote-contact",
 				  G_CALLBACK (chat_remote_contact_changed_cb),
 				  chat);
-	g_signal_connect (tp_chat, "destroy",
-			  G_CALLBACK (chat_destroy_cb),
-			  chat);
 
 	chat_remote_contact_changed_cb (chat);
 
@@ -1614,12 +1701,14 @@ empathy_chat_set_tp_chat (EmpathyChat   *chat,
 		}
 	}
 
-	empathy_tp_chat_set_acknowledge (priv->tp_chat, TRUE);
-	empathy_tp_chat_emit_pendings (priv->tp_chat);
-
 	g_object_notify (G_OBJECT (chat), "tp-chat");
 	g_object_notify (G_OBJECT (chat), "id");
 	g_object_notify (G_OBJECT (chat), "account");
+
+	/* This is a noop when tp-chat is set at object construction time and causes
+	 * the pending messages to be show when it's set on the object after it has
+	 * been created */
+	show_pending_messages (chat);
 }
 
 McAccount *
@@ -1681,20 +1770,6 @@ empathy_chat_get_remote_contact (EmpathyChat *chat)
 	return priv->remote_contact;
 }
 
-guint
-empathy_chat_get_members_count (EmpathyChat *chat)
-{
-	EmpathyChatPriv *priv = GET_PRIV (chat);
-
-	g_return_val_if_fail (EMPATHY_IS_CHAT (chat), 0);
-
-	if (priv->tp_chat) {
-		return empathy_tp_chat_get_members_count (priv->tp_chat);
-	}
-
-	return 0;
-}
-
 GtkWidget *
 empathy_chat_get_contact_menu (EmpathyChat *chat)
 {
