@@ -30,12 +30,13 @@
 #include <glib/gi18n-lib.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
+#include <telepathy-glib/util.h>
 #include <libmissioncontrol/mc-account.h>
 
+#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-call-factory.h>
-#include <libempathy/empathy-contact-factory.h>
+#include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-contact-groups.h>
 #include <libempathy/empathy-dispatcher.h>
@@ -123,8 +124,8 @@ contact_list_view_tooltip_destroy_cb (GtkWidget              *widget,
 	
 	if (priv->tooltip_widget) {
 		DEBUG ("Tooltip destroyed");
+		g_object_unref (priv->tooltip_widget);
 		priv->tooltip_widget = NULL;
-		g_object_unref (widget);
 	}
 }
 
@@ -141,16 +142,19 @@ contact_list_view_query_tooltip_cb (EmpathyContactListView *view,
 	GtkTreeModel               *model;
 	GtkTreeIter                 iter;
 	GtkTreePath                *path;
+	static gint                 running = 0;
+	gboolean                    ret = FALSE;
 
-	/* FIXME: We need GTK version >= 2.12.10. See GNOME bug #504087 */
-	if (gtk_check_version (2, 12, 10)) {
+	/* Avoid an infinite loop. See GNOME bug #574377 */
+	if (running > 0) {
 		return FALSE;
 	}
+	running++;
 
 	if (!gtk_tree_view_get_tooltip_context (GTK_TREE_VIEW (view), &x, &y,
 						keyboard_mode,
 						&model, &path, &iter)) {
-		return FALSE;
+		goto OUT;
 	}
 
 	gtk_tree_view_set_tooltip_row (GTK_TREE_VIEW (view), tooltip, path);
@@ -160,7 +164,7 @@ contact_list_view_query_tooltip_cb (EmpathyContactListView *view,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
 			    -1);
 	if (!contact) {
-		return FALSE;
+		goto OUT;
 	}
 
 	if (!priv->tooltip_widget) {
@@ -170,20 +174,68 @@ contact_list_view_query_tooltip_cb (EmpathyContactListView *view,
 		g_signal_connect (priv->tooltip_widget, "destroy",
 				  G_CALLBACK (contact_list_view_tooltip_destroy_cb),
 				  view);
+		gtk_widget_show (priv->tooltip_widget);
 	} else {
 		empathy_contact_widget_set_contact (priv->tooltip_widget,
 						    contact);
 	}
 
 	gtk_tooltip_set_custom (tooltip, priv->tooltip_widget);
+	ret = TRUE;
 
 	g_object_unref (contact);
+OUT:
+	running--;
 
-	return TRUE;
+	return ret;
+}
+
+typedef struct {
+	gchar *new_group;
+	gchar *old_group;
+	GdkDragAction action;
+} DndGetContactData;
+
+static void
+contact_list_view_dnd_get_contact_free (DndGetContactData *data)
+{
+	g_free (data->new_group);
+	g_free (data->old_group);
+	g_slice_free (DndGetContactData, data);
 }
 
 static void
-contact_list_view_drag_data_received (GtkWidget         *widget,
+contact_list_view_drag_got_contact (EmpathyTpContactFactory *factory,
+				    EmpathyContact          *contact,
+				    const GError            *error,
+				    gpointer                 user_data,
+				    GObject                 *view)
+{
+	EmpathyContactListViewPriv *priv = GET_PRIV (view);
+	DndGetContactData          *data = user_data;
+	EmpathyContactList         *list;
+
+	if (error != NULL) {
+		DEBUG ("Error: %s", error->message);
+		return;
+	}
+
+	DEBUG ("contact %s (%d) dragged from '%s' to '%s'",
+		empathy_contact_get_id (contact),
+		empathy_contact_get_handle (contact),
+		data->old_group, data->new_group);
+
+	list = empathy_contact_list_store_get_list_iface (priv->store);
+	if (data->new_group) {
+		empathy_contact_list_add_to_group (list, contact, data->new_group);
+	}
+	if (data->old_group && data->action == GDK_ACTION_MOVE) {	
+		empathy_contact_list_remove_from_group (list, contact, data->old_group);
+	}
+}
+
+static void
+contact_list_view_drag_data_received (GtkWidget         *view,
 				      GdkDragContext    *context,
 				      gint               x,
 				      gint               y,
@@ -192,20 +244,53 @@ contact_list_view_drag_data_received (GtkWidget         *widget,
 				      guint              time)
 {
 	EmpathyContactListViewPriv *priv;
-	EmpathyContactList         *list;
-	EmpathyContactFactory      *factory;
+	EmpathyAccountManager      *account_manager;
+	EmpathyTpContactFactory    *factory = NULL;
 	McAccount                  *account;
 	GtkTreeModel               *model;
-	GtkTreePath                *path;
 	GtkTreeViewDropPosition     position;
-	EmpathyContact             *contact = NULL;
+	GtkTreePath                *path;
 	const gchar                *id;
-	gchar                     **strv;
+	gchar                     **strv = NULL;
+	const gchar                *account_id;
+	const gchar                *contact_id;
 	gchar                      *new_group = NULL;
 	gchar                      *old_group = NULL;
+	DndGetContactData          *data;
 	gboolean                    is_row;
+	gboolean                    success = TRUE;
 
-	priv = GET_PRIV (widget);
+	priv = GET_PRIV (view);
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
+
+	/* Get destination group information. */
+	is_row = gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (view),
+						    x,
+						    y,
+						    &path,
+						    &position);
+
+	if (is_row) {
+		new_group = empathy_contact_list_store_get_parent_group (model,
+			path, NULL);
+		gtk_tree_path_free (path);
+	}
+
+	/* Get source group information. */
+	if (priv->drag_row) {
+		path = gtk_tree_row_reference_get_path (priv->drag_row);
+		if (path) {
+			old_group = empathy_contact_list_store_get_parent_group (
+				model, path, NULL);
+			gtk_tree_path_free (path);
+		}
+	}
+
+	if (!tp_strdiff (old_group, new_group)) {
+		g_free (new_group);
+		g_free (old_group);
+		goto OUT;
+	}
 
 	id = (const gchar*) selection->data;
 	DEBUG ("Received %s%s drag & drop contact from roster with id:'%s'",
@@ -214,66 +299,47 @@ contact_list_view_drag_data_received (GtkWidget         *widget,
 		id);
 
 	strv = g_strsplit (id, "/", 2);
-	factory = empathy_contact_factory_dup_singleton ();
-	account = mc_account_lookup (strv[0]);
+	account_id = strv[0];
+	contact_id = strv[1];
+	account = mc_account_lookup (account_id);
 	if (account) {
-		contact = empathy_contact_factory_get_from_id (factory,
-							       account,
-							       strv[1]);
-		g_object_unref (account);
-	}
-	g_object_unref (factory);
-	g_strfreev (strv);
+		TpConnection *connection;
 
-	if (!contact) {
-		DEBUG ("No contact found associated with drag & drop");
-		return;
-	}
-
-	empathy_contact_run_until_ready (contact,
-					 EMPATHY_CONTACT_READY_HANDLE,
-					 NULL);
-
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
-
-	/* Get source group information. */
-	if (priv->drag_row) {
-		path = gtk_tree_row_reference_get_path (priv->drag_row);
-		if (path) {
-			old_group = empathy_contact_list_store_get_parent_group (model, path, NULL);
-			gtk_tree_path_free (path);
+		/* FIXME: We assume we have already an account manager */
+		account_manager = empathy_account_manager_dup_singleton ();
+		connection = empathy_account_manager_get_connection (account_manager,
+								     account);
+		if (connection) {
+			factory = empathy_tp_contact_factory_dup_singleton (connection);
 		}
+		g_object_unref (account_manager);
 	}
 
-	/* Get destination group information. */
-	is_row = gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (widget),
-						    x,
-						    y,
-						    &path,
-						    &position);
-
-	if (is_row) {
-		new_group = empathy_contact_list_store_get_parent_group (model, path, NULL);
-		gtk_tree_path_free (path);
+	if (!factory) {
+		DEBUG ("Failed to get factory for account '%s'", account_id);
+		success = FALSE;
+		g_free (new_group);
+		g_free (old_group);
+		goto OUT;
 	}
 
-	DEBUG ("contact %s (%d) dragged from '%s' to '%s'",
-		empathy_contact_get_id (contact),
-		empathy_contact_get_handle (contact),
-		old_group, new_group);
+	data = g_slice_new0 (DndGetContactData);
+	data->new_group = new_group;
+	data->old_group = old_group;
+	data->action = context->action;
 
-	list = empathy_contact_list_store_get_list_iface (priv->store);
-	if (new_group) {
-		empathy_contact_list_add_to_group (list, contact, new_group);
-	}
-	if (old_group && context->action == GDK_ACTION_MOVE) {	
-		empathy_contact_list_remove_from_group (list, contact, old_group);
-	}
+	/* FIXME: We should probably wait for the cb before calling
+	 * gtk_drag_finish */
+	empathy_tp_contact_factory_get_from_id (factory, contact_id,
+		contact_list_view_drag_got_contact,
+		data, (GDestroyNotify) contact_list_view_dnd_get_contact_free,
+		G_OBJECT (view));
 
-	g_free (old_group);
-	g_free (new_group);
+	g_object_unref (factory);
 
-	gtk_drag_finish (context, TRUE, FALSE, GDK_CURRENT_TIME);
+OUT:
+	g_strfreev (strv);
+	gtk_drag_finish (context, success, FALSE, GDK_CURRENT_TIME);
 }
 
 static gboolean
@@ -408,7 +474,7 @@ contact_list_view_drag_data_get (GtkWidget        *widget,
 
 	gtk_tree_path_free (src_path);
 
-	contact = empathy_contact_list_view_get_selected (EMPATHY_CONTACT_LIST_VIEW (widget));
+	contact = empathy_contact_list_view_dup_selected (EMPATHY_CONTACT_LIST_VIEW (widget));
 	if (!contact) {
 		return;
 	}
@@ -1152,7 +1218,7 @@ empathy_contact_list_view_new (EmpathyContactListStore        *store,
 }
 
 EmpathyContact *
-empathy_contact_list_view_get_selected (EmpathyContactListView *view)
+empathy_contact_list_view_dup_selected (EmpathyContactListView *view)
 {
 	EmpathyContactListViewPriv *priv;
 	GtkTreeSelection          *selection;
@@ -1318,7 +1384,7 @@ contact_list_view_remove_activate_cb (GtkMenuItem            *menuitem,
 	EmpathyContactListViewPriv *priv = GET_PRIV (view);
 	EmpathyContact             *contact;
 		
-	contact = empathy_contact_list_view_get_selected (view);
+	contact = empathy_contact_list_view_dup_selected (view);
 
 	if (contact) {
 		gchar     *text; 
@@ -1351,7 +1417,7 @@ empathy_contact_list_view_get_contact_menu (EmpathyContactListView *view)
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
 
-	contact = empathy_contact_list_view_get_selected (view);
+	contact = empathy_contact_list_view_dup_selected (view);
 	if (!contact) {
 		return NULL;
 	}
