@@ -158,7 +158,7 @@ empathy_contact_call_menu_item_new (EmpathyContact *contact)
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
-	item = gtk_image_menu_item_new_with_mnemonic (_("_Call"));
+	item = gtk_image_menu_item_new_with_mnemonic (C_("menu item", "_Call"));
 	image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_VOIP,
 					      GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
@@ -236,7 +236,7 @@ empathy_contact_file_transfer_menu_item_new (EmpathyContact *contact)
 static void
 contact_info_menu_item_activate_cb (EmpathyContact *contact)
 {
-	empathy_contact_information_dialog_show (contact, NULL, FALSE, FALSE);
+	empathy_contact_information_dialog_show (contact, NULL);
 }
 
 GtkWidget *
@@ -263,7 +263,7 @@ empathy_contact_info_menu_item_new (EmpathyContact *contact)
 static void
 contact_edit_menu_item_activate_cb (EmpathyContact *contact)
 {
-	empathy_contact_information_dialog_show (contact, NULL, TRUE, FALSE);
+	empathy_contact_edit_dialog_show (contact, NULL);
 }
 
 GtkWidget *
